/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.utils;

import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH;
import static org.fortiss.tooling.base.layout.LayoutKeyConstants.CONNECTION_POINTS;
import static org.fortiss.tooling.base.layout.LayoutKeyConstants.CONNECTOR_DIMENSION;
import static org.fortiss.tooling.base.layout.LayoutKeyConstants.CONNECTOR_POSITION;
import static org.fortiss.tooling.base.layout.LayoutKeyConstants.NODE_DIMENSION;
import static org.fortiss.tooling.base.layout.LayoutKeyConstants.NODE_POSITION;
import static org.fortiss.tooling.base.utils.DimensionUtils.setDimension;
import static org.fortiss.tooling.base.utils.PointUtils.setPoint;

import org.fortiss.tooling.base.model.layout.Angle;
import org.fortiss.tooling.base.model.layout.Dimension;
import org.fortiss.tooling.base.model.layout.EOrientation;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.LayoutFactory;
import org.fortiss.tooling.base.model.layout.OffsetOrientation;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.model.layout.Points;
import org.fortiss.tooling.base.model.layout.Rectangle;

/**
 * This class provides the creation methods for model elements.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: E17B0746F5AB2F379B8F36F2135DA13F
 */
public final class LayoutModelElementFactory {

	/** Creates a model element node layout. */
	public static void createNodeElementLayout(ILayoutedModelElement layouted) {
		setPoint(layouted, NODE_POSITION, 0, 0);

		setDimension(layouted, NODE_DIMENSION, DEFAULT_SHAPE_MINIMUM_WIDTH,
				DEFAULT_SHAPE_MINIMUM_HEIGHT);
	}

	/** Creates a connector layout. */
	public static void createConnectorLayout(ILayoutedModelElement layouted) {
		setPoint(layouted, NODE_POSITION, 0, 0);

		setDimension(layouted, NODE_DIMENSION, DEFAULT_SHAPE_MINIMUM_WIDTH,
				DEFAULT_SHAPE_MINIMUM_HEIGHT);

		Point cpos = createPoint(0, 0, CONNECTOR_POSITION);

		Dimension cdim = createDimension(0, 0, CONNECTOR_DIMENSION);

		layouted.getLayoutData().add(cpos);
		layouted.getLayoutData().add(cdim);
	}

	/** Creates a new connection layout. */
	public static void createConnectionLayout(ILayoutedModelElement layouted) {
		Points points = createPoints(CONNECTION_POINTS);
		layouted.getLayoutData().add(points);
	}

	/** Creates a new {@link Angle}. */
	public static Angle createAngle(double angle, String key) {
		Angle a = LayoutFactory.eINSTANCE.createAngle();
		a.setAngle(angle);
		a.setKey(key);
		return a;
	}

	/** Creates a new {@link Dimension}. */
	public static Dimension createDimension(int w, int h, String key) {
		Dimension d = LayoutFactory.eINSTANCE.createDimension();
		d.setWidth(w);
		d.setHeight(h);
		d.setKey(key);
		return d;
	}

	/** Creates a new {@link Points} layout data. */
	public static Points createPoints(String key) {
		Points pts = LayoutFactory.eINSTANCE.createPoints();
		pts.setKey(key);
		return pts;
	}

	/** Creates a new {@link Point}. */
	public static Point createPoint(int x, int y, String key) {
		Point p = LayoutFactory.eINSTANCE.createPoint();
		p.setX(x);
		p.setY(y);
		p.setKey(key);
		return p;
	}

	/** Returns a new {@link OffsetOrientation} object with the given values. */
	public static OffsetOrientation createOffsetOrientation(int offset, EOrientation orientation,
			String key) {
		OffsetOrientation oo = LayoutFactory.eINSTANCE.createOffsetOrientation();
		oo.setOrientation(orientation);
		oo.setOffset(offset);
		oo.setKey(key);
		return oo;
	}

	/** Returns a new {@link Rectangle} with the given parameters. */
	public static Rectangle createRectangle(int x, int y, int width, int height) {
		Rectangle rectangle = LayoutFactory.eINSTANCE.createRectangle();
		rectangle.setLocation(x, y);
		rectangle.setSize(width, height);
		return rectangle;
	}
}
