/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension;

import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.extension.IConstraintVerifier;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;

/**
 * Interface for the GUI of constraint verifiers.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 8B7EC3F9AB4256EC702DE0CD91860C09
 */
public interface IConstraintVerifierUI {
	// TODO: should not be needed if the service handles IDs internally
	/**
	 * Returns the unique identifier of this constraint. Should match the corresponding
	 * {@link IConstraintVerifier}!
	 */
	String getID();

	// TODO: remove @params
	/**
	 * Action to take when trying to open the given constraint. <code>canOpen</code> should be
	 * called before to know if the status can be opened.
	 * 
	 * @param constraint
	 * @param status
	 * @return true if the constraint could indeed be open, false if it needs further handling.
	 */
	public boolean openStatus(ConstraintInstance constraint, IConstraintVerificationStatus status);

	/** True if the constraint can be open. */
	boolean canOpen(ConstraintInstance constraint, IConstraintVerificationStatus status);

	/** Method to execute when a constraint gets outdated. */
	public void onOutdate(ConstraintInstance constraint);

	/** Returns a short user-friendly description explaining the constraint. */
	public String getMessage(ConstraintInstance constraint, IConstraintVerificationStatus status);

	/**
	 * Set to <code>true</code> if you want your constraint to be displayed as a warning rather than
	 * as an error.
	 */
	public boolean displayAsWarning();

	/** Gets a general description for the constraint type. */
	String getDescription();

	/** Returns the image descriptor to be used as icon image. */
	public ImageDescriptor getIconImageDescriptor();

	/**
	 * Way for the verifier to indicate that the verification is time consuming.
	 * This triggers a few goodies:
	 * - wait 1s before triggering the constraint autocheck (if relevant)
	 * - display a warning popup in case of first trigger of the verification
	 */
	public boolean isTimeConsuming();

	/**
	 * True if this constraint shall be always activated (and therefore the user shall not be
	 * offered the possibility to control whether the constraint is active or not).
	 */
	public boolean shouldBeManuallyActivated();
}
