/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.ui.extension.IConstraintVerifierUI;
import org.fortiss.tooling.kernel.ui.internal.introspection.items.ConstraintVerificationUIServiceIntrospectionDetailsItem;

/**
 * Introspection UI handler for the {@link IConnectionCompositorService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: C3E3F04E22A933512334608C48F361FC
 */
public final class ConstraintVerificationUIServiceIntrospectionDetailsUIHandler extends
		EObjectAwareIntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		String heading = "Type to search registered constraint verifier UI's:";
		return createFilteredTreeInTabFolder(parent, heading, "", "Registered verifier UI's");
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("ID / Verifier UI");
		col0.setWidth(300);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Verifier UI Class");
		col1.setWidth(600);
	}

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new TreeContentProviderBase() {

			/** {@inheritDoc} */
			@Override
			public Object[] getChildren(Object parentElement) {
				if(parentElement == getInputObject()) {
					return getInputObject().getHandlerKeyClasses().toArray();
				}
				if(parentElement instanceof String) {
					String regClass = (String)parentElement;
					IConstraintVerifierUI handler = getInputObject().getHandler(regClass);
					if(handler == null) {
						return new Object[0];
					}
					IConstraintVerifierUI[] kids = new IConstraintVerifierUI[1];
					kids[0] = handler;
					return kids;
				}
				return new Object[0];
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new TableLabelProvider();
	}

	/** Table label provider for handler registrations with two classes. */
	protected static class TableLabelProvider extends TableLabelProviderBase {
		/** {@inheritDoc} */
		@Override
		public String getColumnText(Object element, int columnIndex) {
			if(element instanceof String) {
				return columnIndex == 0 ? "\"" + (String)element + "\"" : "";
			}
			if(element instanceof IConstraintVerifierUI) {
				IConstraintVerifierUI provider = (IConstraintVerifierUI)element;
				switch(columnIndex) {
					case 0:
						return provider.getClass().getSimpleName();
					case 1:
						return provider.getClass().getName();
				}
			}
			return "";
		}
	}

	/** {@inheritDoc} */
	@Override
	protected ConstraintVerificationUIServiceIntrospectionDetailsItem getInputObject() {
		return (ConstraintVerificationUIServiceIntrospectionDetailsItem)dataItem;
	}
}
