package org.fortiss.tooling.kernel.ui.service;

import java.util.Set;

import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstraintInstanceContainer;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.ui.extension.IConstraintVerifierUI;
import org.fortiss.tooling.kernel.ui.internal.ConstraintVerificationUIService;

/**
 * GUI aspects of {@link IConstraintVerificationService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D43B85C6B904F01588C5A69234572718
 */
public interface IConstraintVerificationUIService {
	/** Returns the service instance. */
	public static IConstraintVerificationUIService getInstance() {
		return ConstraintVerificationUIService.getInstance();
	}

	/** Action to take when trying to open the (status of the given) constraint instance. */
	void openStatus(ConstraintInstance ci);

	/** True if the (status of the) the given constraint instance can be open. */
	boolean canOpen(ConstraintInstance ci);

	/** Install the given constraint instance: install notifiers and triggers an "onOutdate" event. */
	void install(ConstraintInstance ci);

	/**
	 * True if a failure of the given constraint shall be displayed as a warning instead of an
	 * error.
	 */
	boolean shallDisplayAsWarning(ConstraintInstance ci);

	/** Returns a description for the given constraint type. */
	// TODO: use ConstraintInstance and call getConstraintID() internally
	// otherwise this String looks very arbitrary
	String getDescription(String id);

	/** Returns the image descriptor to be used as icon image. */
	// TODO: use ConstraintInstance and call getConstraintID() internally
	// otherwise this String looks very arbitrary
	ImageDescriptor getIconImageDescriptor(String id);

	/**
	 * Browse the elements in the context of the given container, adds constraints instances if
	 * relevant and install them.
	 */
	void addMissingConstraintInstances(IConstraintInstanceContainer cstrContainer);

	/**
	 * Returns the status of the constraint instance, possibly returns <code>null</code> if the
	 * constraint instance is not activated.
	 */
	IConstraintVerificationStatus getStatus(ConstraintInstance ci);

	/** Returns a list of all available constraints (through their IDs). */
	// TODO: use ConstraintInstance instead of String to be consistent with above changes
	Set<String> getAllConstraints();

	/** Returns a list of constraints which should always be activated (through their IDs). */
	// TODO: use ConstraintInstance instead of String to be consistent with above changes
	Set<String> getAlwaysActivatedConstraints();

	/**
	 * Returns a list of all constraints which are activated for the given top element (through
	 * their IDs).
	 */
	Set<String> getActivatedConstraints(ITopLevelElement top);

	/** Activates the constraint in the context of the given constraints container. */
	void activate(String id, IConstraintInstanceContainer cstrContainer);

	/** Deactivates the constraint in the context of the given constraints container. */
	void deactivate(String id, IConstraintInstanceContainer cstrContainer);

	/** Registers the verifier UI part with the kernel. */
	// TODO: change interface (see TutorialServiceUI for implementation).
	// void registerTutorialUIProvider(Class<? extends IConstraintVerifier> nonUIverifier,
	// Class<? extends IConstraintVerifierUI> uiVerifier)
	void registerConstraintVerifierUI(IConstraintVerifierUI verifier);
}
