/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import org.fortiss.tooling.kernel.model.INamedElement;

/**
 * Utility class for name identifiers and unique identifiers.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 5EC8C933CA964DDA4092FDF954A7DE88
 */
public class IdentifierUtils {

	/** The delimiter for id in generated unique identifiers for model elements. */
	public static final String ID_DELIMITER = "_ID_";

	/**
	 * Returns the unique identifier built from the model element's name and ID.
	 * All non-word characters in the returned string are replace with '_'.
	 * 
	 * @param modelElement
	 *            the named model element with an ID
	 * @return string with format &lt;element_name&gt;_ID_&lt;element_id&gt;
	 */
	public static String getUniqueIdentifier(INamedElement modelElement) {
		return (modelElement.getName() + ID_DELIMITER + modelElement.getId())
				.replaceAll("\\W", "_");
	}

	/**
	 * Returns the identifier built from the model element's name. All non-word
	 * characters in the returned string are replace with '_'.
	 * 
	 * @param modelElement
	 *            the named model element with an ID
	 * @return string with format &lt;element_name&gt;
	 */
	public static String getIdentifier(INamedElement modelElement) {
		return modelElement.getName().replaceAll("\\W", "_");
	}

	/**
	 * Returns true if the identifier contains other characters than those allowed by Java program
	 * element names
	 * 
	 * @param modelElement
	 *            the named model element with an ID
	 * @return true if disallowed characters are used.
	 */
	public static boolean containsDisallowedCharactersInName(INamedElement modelElement) {
		String name = modelElement.getName();
		return(Character.isSpaceChar(name.charAt(name.length() - 1)));
	}
}
