/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.databinding;

import org.eclipse.core.databinding.Binding;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.validation.IValidator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.databinding.swt.ISWTObservableValue;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;

/**
 * Observables for JFace stuff.
 * 
 * @author hummelb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: C9DDEEA26B1DDF633067133A9AD6DCEF
 */
public class JFaceObservables {

	/**
	 * Returns a new (write-only) observable for writing an {@link IStatus} into
	 * a JFace {@link ControlDecoration}.
	 */
	public static IObservableValue observeControlDecoration(
			ControlDecoration controlDecoration) {
		return new ControlDecorationObservable(controlDecoration);
	}

	/**
	 * Creates a new binding to a text widget which validates its input.
	 * Validation will be performed on modify and signaled by the decoration,
	 * while actual write through to the model will only happen on focusOut if
	 * validation is ok.
	 * 
	 * 
	 * @param text
	 *            the text widget to bind to.
	 * @param textDecoration
	 *            the decoration of the widget to signal errors to.
	 * @param validator
	 *            the validator used for input validation.
	 * @param modelObservable
	 *            the model observable used for persisting
	 * @param dbc
	 *            the data binding context.
	 */
	public static void setupValidatedTextBinding(Text text,
			ControlDecoration textDecoration, IValidator validator,
			IObservableValue modelObservable, DataBindingContext dbc) {
		ISWTObservableValue nameTextFocusOutObservable = SWTObservables
				.observeText(text, SWT.FocusOut);
		UpdateValueStrategy mainUpdateStrategy = new UpdateValueStrategy()
				.setBeforeSetValidator(validator);
		dbc.bindValue(nameTextFocusOutObservable, modelObservable,
				mainUpdateStrategy, null);

		ISWTObservableValue nameTextModifyObservable = SWTObservables
				.observeText(text, SWT.Modify);
		UpdateValueStrategy decorationUpdateStrategy = new UpdateValueStrategy(
				true, UpdateValueStrategy.POLICY_CONVERT)
				.setAfterGetValidator(validator);
		Binding nameBinding = dbc.bindValue(nameTextModifyObservable,
				modelObservable, decorationUpdateStrategy,
				new UpdateValueStrategy(UpdateValueStrategy.POLICY_NEVER));
		dbc.bindValue(observeControlDecoration(textDecoration), nameBinding
				.getValidationStatus(), null, null);
	}

}